@@ -1,4 +1,4 @@
-# (C) Copyright 2005-2020 Enthought, Inc., Austin, TX
+# (C) Copyright 2005-2021 Enthought, Inc., Austin, TX
 # All rights reserved.
 #
 # This software is provided without warranty under the terms of the BSD
@@ -11,11 +11,8 @@
 """ Classes that implement and support the Traits change notification mechanism
 """
 
-# -------------------------------------------------------------------------------
-#  Imports:
-# -------------------------------------------------------------------------------
-
 import contextlib
+import logging
 import threading
 from threading import local as thread_local
 from threading import Thread
@@ -24,12 +21,11 @@ from types import MethodType
 import weakref
 import sys
 
+from .constants import ComparisonMode, TraitKind
 from .trait_base import Uninitialized
 from .trait_errors import TraitNotificationError
 
-# -------------------------------------------------------------------------------
-#  Global Data:
-# -------------------------------------------------------------------------------
+# Global Data
 
 # The thread ID for the user interface thread
 ui_thread = -1
@@ -37,10 +33,6 @@ ui_thread = -1
 # The handler for notifications that must be run on the UI thread
 ui_handler = None
 
-# -------------------------------------------------------------------------------
-#  Sets up the user interface thread handler:
-# -------------------------------------------------------------------------------
-
 
 def set_ui_handler(handler):
     """ Sets up the user interface thread handler.
@@ -58,11 +50,6 @@ def ui_dispatch(handler, *args, **kw):
         ui_handler(handler, *args, **kw)
 
 
-# -------------------------------------------------------------------------------
-#  'NotificationExceptionHandlerState' class:
-# -------------------------------------------------------------------------------
-
-
 class NotificationExceptionHandlerState(object):
     def __init__(self, handler, reraise_exceptions, locked):
         self.handler = handler
@@ -70,18 +57,13 @@ class NotificationExceptionHandlerState(object):
         self.locked = locked
 
 
-# -------------------------------------------------------------------------------
-#  'NotificationExceptionHandler' class:
-# -------------------------------------------------------------------------------
-
-
 class NotificationExceptionHandler(object):
     def __init__(self):
         self.traits_logger = None
         self.main_thread = None
         self.thread_local = thread_local()
 
-    # -- Private Methods ------------------------------------------------------------
+    # -- Private Methods ------------------------------------------------------
 
     def _push_handler(
         self, handler=None, reraise_exceptions=False, main=False, locked=False
@@ -201,16 +183,12 @@ class NotificationExceptionHandler(object):
                 "No changes are allowed."
             )
 
-    # ---------------------------------------------------------------------------
-    #  This method defines the default notification exception handling
-    #  behavior of traits. However, it can be completely overridden by pushing
-    #  a new handler using the '_push_handler' method.
-    #
-    #  It logs any exceptions generated in a trait notification handler.
-    # ---------------------------------------------------------------------------
-
     def _log_exception(self, object, trait_name, old, new):
         """ Logs any exceptions generated in a trait notification handler.
+
+        This method defines the default notification exception handling
+        behavior of traits. However, it can be completely overridden by pushing
+        a new handler using the '_push_handler' method.
         """
         # When the stack depth is too great, the logger can't always log the
         # message. Make sure that it goes to the console at a minimum:
@@ -235,16 +213,7 @@ class NotificationExceptionHandler(object):
 
         logger = self.traits_logger
         if logger is None:
-            import logging
-
             self.traits_logger = logger = logging.getLogger("traits")
-            handler = logging.StreamHandler()
-            handler.setFormatter(logging.Formatter("%(message)s"))
-            logger.addHandler(handler)
-            print(
-                "Exception occurred in traits notification handler.\n"
-                "Please check the log file for details."
-            )
 
         try:
             logger.exception(
@@ -257,9 +226,7 @@ class NotificationExceptionHandler(object):
             pass
 
 
-# -------------------------------------------------------------------------------
-#  Traits global notification exception handler:
-# -------------------------------------------------------------------------------
+# Traits global notification exception handler
 
 notification_exception_handler = NotificationExceptionHandler()
 
@@ -267,9 +234,7 @@ push_exception_handler = notification_exception_handler._push_handler
 pop_exception_handler = notification_exception_handler._pop_handler
 handle_exception = notification_exception_handler._handle_exception
 
-# -------------------------------------------------------------------------------
-#  Traits global notification event tracer:
-# -------------------------------------------------------------------------------
+# Traits global notification event tracer
 
 _pre_change_event_tracer = None
 _post_change_event_tracer = None
@@ -333,11 +298,6 @@ def change_event_tracers(pre_tracer, post_tracer):
         set_change_event_tracers(old_pre_tracer, old_post_tracer)
 
 
-# -------------------------------------------------------------------------------
-#  'AbstractStaticChangeNotifyWrapper' class:
-# -------------------------------------------------------------------------------
-
-
 class AbstractStaticChangeNotifyWrapper(object):
     """
     Concrete implementation must define the 'argument_transforms' class
@@ -353,9 +313,9 @@ class AbstractStaticChangeNotifyWrapper(object):
         if arg_count > 4:
             raise TraitNotificationError(
                 (
-                    "Invalid number of arguments for the static anytrait change "
-                    "notification handler: %s. A maximum of 4 arguments is "
-                    "allowed, but %s were specified."
+                    "Invalid number of arguments for the static anytrait "
+                    "change notification handler: %s. A maximum of 4 "
+                    "arguments is allowed, but %s were specified."
                 )
                 % (handler.__name__, arg_count)
             )
@@ -366,7 +326,8 @@ class AbstractStaticChangeNotifyWrapper(object):
     def __call__(self, object, trait_name, old, new):
         """ Dispatch to the appropriate handler method. """
 
-        if old is not Uninitialized:
+        if _change_accepted(object, trait_name, old, new):
+
             # Extract the arguments needed from the handler.
             args = self.argument_transform(object, trait_name, old, new)
 
@@ -400,11 +361,6 @@ class AbstractStaticChangeNotifyWrapper(object):
         return False
 
 
-# -------------------------------------------------------------------------------
-#  'StaticAnyTraitChangeNotifyWrapper' class:
-# -------------------------------------------------------------------------------
-
-
 class StaticAnyTraitChangeNotifyWrapper(AbstractStaticChangeNotifyWrapper):
 
     # The wrapper is called with the full set of argument, and we need to
@@ -419,11 +375,6 @@ class StaticAnyTraitChangeNotifyWrapper(AbstractStaticChangeNotifyWrapper):
     }
 
 
-# -------------------------------------------------------------------------------
-#  'StaticTraitChangeNotifyWrapper' class:
-# -------------------------------------------------------------------------------
-
-
 class StaticTraitChangeNotifyWrapper(AbstractStaticChangeNotifyWrapper):
 
     # The wrapper is called with the full set of argument, and we need to
@@ -438,11 +389,6 @@ class StaticTraitChangeNotifyWrapper(AbstractStaticChangeNotifyWrapper):
     }
 
 
-# -------------------------------------------------------------------------------
-#  'TraitChangeNotifyWrapper' class:
-# -------------------------------------------------------------------------------
-
-
 class TraitChangeNotifyWrapper(object):
     """ Dynamic change notify wrapper.
 
@@ -479,9 +425,9 @@ class TraitChangeNotifyWrapper(object):
                 if arg_count > 4:
                     raise TraitNotificationError(
                         (
-                            "Invalid number of arguments for the dynamic trait "
-                            "change notification handler: %s. A maximum of 4 "
-                            "arguments is allowed, but %s were specified."
+                            "Invalid number of arguments for the dynamic "
+                            "trait change notification handler: %s. A maximum "
+                            "of 4 arguments is allowed, but %s were specified."
                         )
                         % (func.__name__, arg_count)
                     )
@@ -594,7 +540,8 @@ class TraitChangeNotifyWrapper(object):
         """ Dispatch a trait change event to a method listener. """
 
         obj_weak_ref = self.object
-        if (obj_weak_ref is not None) and (old is not Uninitialized):
+        if (obj_weak_ref is not None
+                and _change_accepted(object, trait_name, old, new)):
             # We make sure to hold a reference to the object before invoking
             # `getattr` so that the listener does not disappear in a
             # multi-threaded case.
@@ -609,17 +556,12 @@ class TraitChangeNotifyWrapper(object):
     def _notify_function_listener(self, object, trait_name, old, new):
         """ Dispatch a trait change event to a function listener. """
 
-        if old is not Uninitialized:
+        if _change_accepted(object, trait_name, old, new):
             self._dispatch_change_event(
                 object, trait_name, old, new, self.handler
             )
 
 
-# -------------------------------------------------------------------------------
-#  'ExtendedTraitChangeNotifyWrapper' class:
-# -------------------------------------------------------------------------------
-
-
 class ExtendedTraitChangeNotifyWrapper(TraitChangeNotifyWrapper):
     """ Change notify wrapper for "extended" trait change events..
 
@@ -664,11 +606,6 @@ class ExtendedTraitChangeNotifyWrapper(TraitChangeNotifyWrapper):
         self._dispatch_change_event(object, trait_name, old, new, self.handler)
 
 
-# -------------------------------------------------------------------------------
-#  'FastUITraitChangeNotifyWrapper' class:
-# -------------------------------------------------------------------------------
-
-
 class FastUITraitChangeNotifyWrapper(TraitChangeNotifyWrapper):
     """ Dynamic change notify wrapper, dispatching on the UI thread.
 
@@ -684,11 +621,6 @@ class FastUITraitChangeNotifyWrapper(TraitChangeNotifyWrapper):
             ui_handler(handler, *args)
 
 
-# -------------------------------------------------------------------------------
-#  'NewTraitChangeNotifyWrapper' class:
-# -------------------------------------------------------------------------------
-
-
 class NewTraitChangeNotifyWrapper(TraitChangeNotifyWrapper):
     """ Dynamic change notify wrapper, dispatching on a new thread.
 
@@ -699,3 +631,37 @@ class NewTraitChangeNotifyWrapper(TraitChangeNotifyWrapper):
 
     def dispatch(self, handler, *args):
         Thread(target=handler, args=args).start()
+
+
+def _change_accepted(object, name, old, new):
+    """ Return true if notifications should be emitted for the change.
+
+    Parameters
+    ----------
+    object : HasTraits
+        The object on which the trait is changed.
+    name : str
+        The name of the trait changed.
+    old : any
+        The old value
+    new : any
+        The new value
+
+    Returns
+    -------
+    accepted : bool
+        Whether the event should be emitted.
+    """
+    if old is Uninitialized:
+        return False
+
+    trait = object._trait(name, 2)
+    if (trait.type == TraitKind.trait.name
+            and trait.comparison_mode == ComparisonMode.equality):
+        try:
+            return bool(old != new)
+        except Exception:
+            # Maybe do something else about the exception
+            # enthought/traits#1230
+            pass
+    return True
